/*
	$Id: CppCFData.cpp,v 1.5 2002/11/29 12:29:10 phonohawk Exp $
	---------
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CppCFData.h"

CppCFData::CppCFData()
{
	cf_data = CFDataCreateMutable(NULL,0);
}

CppCFData::CppCFData(CFDataRef copy_source)
{
	cf_data = CFDataCreateMutableCopy(NULL,0,copy_source);
}

CppCFData::CppCFData(const CppCFData& copy_source)
{
	cf_data = CFDataCreateMutableCopy(NULL,0,copy_source.getData());
}

CppCFData::CppCFData(const void* data,unsigned size)
{
	cf_data = CFDataCreateMutable(NULL,0);
	CFDataAppendBytes(cf_data,reinterpret_cast<const UInt8*>(data),static_cast<CFIndex>(size));
}

CppCFData::~CppCFData()
{
	CFRelease(cf_data);
}
		
unsigned CppCFData::getLength() const
{
	return CFDataGetLength(cf_data);
}

unsigned CppCFData::size() const
{
	return CFDataGetLength(cf_data);
}

CppCFData& CppCFData::setLength(unsigned len)
{
	CFDataSetLength(cf_data,len);
	return *this;
}

CFDataRef CppCFData::getData() const
{
	return cf_data;
}

CFDataRef CppCFData::getCloneOfData() const
{
	return CFDataCreateCopy(NULL,cf_data);
}

CppCFData& CppCFData::append(SInt16 data)
{
	CFDataAppendBytes(cf_data,reinterpret_cast<UInt8*>(&data),sizeof(SInt16));
	return *this;
}

CppCFData& CppCFData::append(UInt16 data)
{
	CFDataAppendBytes(cf_data,reinterpret_cast<UInt8*>(&data),sizeof(UInt16));
	return *this;
}

CppCFData& CppCFData::append(CFDataRef data)
{
	CFDataAppendBytes(cf_data,CFDataGetBytePtr(data),CFDataGetLength(data));
	return *this;
}

CppCFData& CppCFData::append(const CppCFData& data)
{
	CFDataAppendBytes(cf_data,CFDataGetBytePtr(data.getData()),CFDataGetLength(data.getData()));
	return *this;
}

CppCFData& CppCFData::append(const void* data,unsigned size)
{
	CFDataAppendBytes(cf_data,reinterpret_cast<const UInt8*>(data),static_cast<CFIndex>(size));
	return *this;
}

SInt16 CppCFData::getSInt16(unsigned pos) const
{
	const UInt8* ptr = CFDataGetBytePtr(cf_data);
	return *( reinterpret_cast<const SInt16*>(&ptr[pos]) );
}

UInt16 CppCFData::getUInt16(unsigned pos) const
{
	const UInt8* ptr = CFDataGetBytePtr(cf_data);
	return *( reinterpret_cast<const UInt16*>(&ptr[pos]) );
}

const char* CppCFData::getDataPtr(unsigned begin_pos) const
{
	const UInt8* ptr = CFDataGetBytePtr(cf_data);
	return reinterpret_cast<const char*>(&ptr[begin_pos]);
}

CppCFData& CppCFData::copyData(unsigned pos,unsigned size,void* dest) const
{
	CFDataGetBytes(cf_data,CFRangeMake(pos,size),reinterpret_cast<UInt8*>(dest));
	return *const_cast<CppCFData*>(this);
}

CppCFData CppCFData::subdata(unsigned start_pos) const
{
	return CppCFData(getDataPtr(start_pos),size() - start_pos);
}

CppCFData CppCFData::subdata(unsigned start_pos,unsigned end_pos) const
{
	return CppCFData(getDataPtr(start_pos),end_pos - start_pos);
}

CppCFData& CppCFData::operator=(const CppCFData& data)
{
	setLength(0);
	append(data);
	return *this;
}
		
CppCFData& CppCFData::operator+=(SInt16 data)
{
	append(data);
	return *this;
}

CppCFData& CppCFData::operator+=(UInt16 data)
{
	append(data);
	return *this;
}

CppCFData& CppCFData::operator+=(const CppCFData& data)
{
	append(data);
	return *this;
}