/*  $Id: KanaTreeController.cpp,v 1.13 2004/01/21 08:34:09 phonohawk Exp $
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002-2004 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include "KanaTreeController.h"
#include "BIMClientServer.h"
#include "CppCFString.h"
#include "SkkConfig.h"

#if 0
#	define DEBUG 1
#endif

#ifdef DEBUG
#	define D_PRINTF printf
#else
#	define D_PRINTF if(0)printf
#endif

KanaTreeController::KanaTreeController(const string& fpath)
    :root("","","",0x00) { // [gȂ̂ŃxBԂܘ_B
    D_PRINTF("rule-file: %s\n",fpath.c_str());
    
    ifstream ifs(fpath.c_str(),ios::in);
    if (!ifs.fail()) {
	load(ifs);
	ifs.close();
    }
}

KanaTreeController::~KanaTreeController() {
    
}

void KanaTreeController::load(istream& is) {
    CppCFString empty;
    
    string str_line;
    while (!is.eof()) {
	getline(is,str_line,'\n');
	if (str_line.length() == 0) continue;
	if (str_line[0] == '#') continue;
	
	CppCFString line(str_line.c_str(),kCFStringEncodingEUC_JP);
	vector<CppCFString> items = line.split(',');
	
	if (items.size() < 4) {
	    // ُȍsB
	    printf("AquaSKK-KanaTreeController: Illegal declaration of rule.\n");
	    printf("   %s\n",str_line.c_str());
	    continue;
	}
	
	// ʒu߂鍀ڂlocationƂƁAlocation̐擪Ō-1Ԗڂ܂ł̎}Sđ݂邱ƂmFA΍B
	CppCFString location = items[0].replace("&comma;",",");
	unsigned len_location = location.length();
	KanaTreeElement* current_branch = &root;
	for (unsigned i = 0;i < len_location - 1;i++) {
	    try {
		current_branch = &(current_branch->trace(location[i]));
	    }
	    catch (ChildNotFoundException& e) {
		// ̎}̂ōB
		KanaTreeElement branch(empty,empty,empty,location[i]);
		
		current_branch->addChild(branch);
		current_branch = &(current_branch->trace(location[i])); // &branchł͑ʖځB
	    }
	}
	
	// tAԂ牺B
	KanaTreeElement leaf(items[1],items[2],items[3],
			     location[len_location-1],
			     (items.size() >= 5 && items[4].length() > 0 ?
			      (items[4])[0] : 0x00));
	current_branch->addChild(leaf);
    }
}

KanaTreeElement& KanaTreeController::trace(CppCFString& queue,KanaTreeElement& parent,unsigned level) {
    try {
	KanaTreeElement& child = parent.trace(queue[level]);
	
#ifdef DEBUG
	CppCFString childs_str;
	for (vector<KanaTreeElement>::const_iterator ite = child.getChilds().begin();
	     ite != child.getChilds().end();
	     ite++) {
	    childs_str.append(',').append(ite->getLabel());
	}
	if (childs_str.length() > 0) childs_str.eraseFirst(1);
	cout << "childs: " << childs_str.toCString() << endl;
#endif
	
	// ONȂ
	D_PRINTF("num of childs: %ld , at level %d , queue length:%d\n",child.getChilds().size(),level,queue.length());
	if (!child.hasAnyChilds()) {
	    // ȏHȂB
	    queue.eraseFirst(level + 1);
	    if (child.getNextState() != 0x00) {
		queue.append(child.getNextState());
	    }
	    return child;
	}
	else {
	    if (level >= queue.length() - 1) {
		// ȏHɂւ炸L[̏IɗĂ܂B
		throw ReachedEndOfQueueException();
	    }
	    else {
		// ċAIɃg[X𑱂B
		return trace(queue,child,level + 1);
	    }
	}
    }
    catch (ChildNotFoundException& e) {
	D_PRINTF("ChildNotFoundException :level=%d\n",level);
	
	// ȏHȂB
	queue.eraseFirst(level);
	if (parent.getNextState() != 0x00) {
	    queue.append(parent.getNextState());
	}
	return parent;
    }
}

CppCFString KanaTreeController::trace(int output_type,CppCFString& queue) {
    CppCFString result;
    while (queue.length() > 0) {
	try {
	    root.trace(queue[0]);
	}
	catch (ChildNotFoundException& e) {
	    // rootqueue[0]̗vf݂Ȃꍇqueuê̂ԂB̎queue͋ɂB
	    result += queue;
	    queue.clear();
	    break;
	}
	
	try {
	    KanaTreeElement& leaf = trace(queue,root);
	    
	    switch (output_type) {
	    case OUTPUT_HIRAGANA:
		result += leaf.getContentAsHiragana();
		break;
		
	    case OUTPUT_ZENKATA:
		result += leaf.getContentAsZenKata();
		break;
		
	    case OUTPUT_HANKATA:
		result += leaf.getContentAsHanKata();
		break;
	    }
	}
	catch (ReachedEndOfQueueException& e) {
	    D_PRINTF("ReachedEndOfQueueException\n");
	    // [v𔲂
	    break;
	}
    }
    return result;
}

KanaTreeController& KanaTreeController::defaultTreeController() {
    static KanaTreeController* _shared_default = NULL;
    if (_shared_default == NULL) {
	// ܂~/Library/AquaSKK/kana-rule-listǂ߂邩ǂ𒲂ׂB
	string user_rule_list_file = SkkConfig::home() + "/Library/AquaSKK/kana-rule-list";
	ifstream ifs(user_rule_list_file.c_str(),ios::in);
	if (!ifs.fail()) {
	    // ǂݍ݉\B
	    ifs.close();
	    _shared_default = new KanaTreeController(user_rule_list_file);
	}
	else {
	    CFBundleRef component_bundle = CFBundleGetBundleWithIdentifier(CFSTR(kIMBundleIdentifier));
	    
	    // ftHg[t@Cւ̃pXURLƂč쐬
	    CFURLRef rulefile_url = NULL;
	    if (component_bundle) {
		rulefile_url = CFBundleCopyResourceURL(component_bundle,CFSTR("kana-rule-list"),NULL,NULL);
	    }
	    
	    if (rulefile_url) {
		// URLtpXAt@CJă[hB
		CFStringRef path_cfstr = CFURLCopyFileSystemPath(rulefile_url,kCFURLPOSIXPathStyle);
		
		CppCFString path(path_cfstr);
		_shared_default = new KanaTreeController(path.toCString());
		
		CFRelease(path_cfstr);
	    }
	    else {
		// [hoȂ̂ŁAKanaTreeController쐬B
		_shared_default = new KanaTreeController("");
	    }
	    
	    if (rulefile_url) CFRelease(rulefile_url);
	}
    }
    return *_shared_default;
}
