/*  $Id: KanjiConversionMode.cpp,v 1.38 2004/01/21 13:04:30 phonohawk Exp $
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002-2004 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "BIMInputEvents.h"
#include "KanjiConversionMode.h"
#include "HiraganaInputMode.h"
#include "ZenKataInputMode.h"
#include "HanKataInputMode.h"
#include "ParentInputMode.h"
#include "CppCFData.h"
#include "WordRegisterMode.h"
#include "ServerConnectionFactory.h"
#include "ClientConfiguration.h"
#include "SkkConfig.h"
#include "KanaTreeController.h"

#if 0
#	define D_PRINTF printf
#else
#	define D_PRINTF if(0)printf
#endif

/*
  ȓ̓[hő啶̃At@xbg͂ƁA
  ̓[hɐ؂ւA̎̕handleInputɓnB
  
  handleInput͂܂擪ɓĂAɂHiraganaInputModeɓnB
  Xy[XϊB艼͂ĂϊB
  
  Oku    
  OkuR   *r
  OkuRi  *  
  
  ̓[h̎́A͂ꂽ͂̂܂KanjiConversionModeɓnB
*/

KanjiConversionMode::KanjiConversionMode(ParentInputMode& parent)
    :ChildInputMode(parent),
     word_register_mode(NULL),
     status(STATUS_NULL),
     show_cands_window_after_Nth_cand(5),
     completion_mode(false),
     henkanModeStatus(false) {
    
}

KanjiConversionMode::~KanjiConversionMode() {
    if (word_register_mode) delete word_register_mode;
}

bool KanjiConversionMode::handleInput(unsigned char inputChar) {
    bool result = handleInputChar(inputChar);
    henkanModeStatus = !result;
    
    return result;
}

bool KanjiConversionMode::handleInputChar(unsigned char inputChar) {
    if (word_register_mode) {
	word_register_mode->handleInput(inputChar);
	return false; // WordRegisterMode::handleInput()ԂƂm͂ȂB
    }
    
    if (SkkConfig::config().isAbbrevNextKey(inputChar)) {
	if (status == STATUS_WHITE_ROOT && completion_mode) {
	    // ̌ցBŌ܂ōsŏɖ߂B
	    current_completion_index++;
	    if (current_completion_index >= completions.size()) {
		current_completion_index = 0;
	    }
	    root = completions[current_completion_index];
	    root_to_display = root;
	    return false;
	}
    }
    
    if (SkkConfig::config().isAbbrevPrevKey(inputChar)) {
	if (status == STATUS_WHITE_ROOT && completion_mode) {
	    // ǑցBŏ܂ōsŌɖ߂B
	    if (current_completion_index == 0) {
		current_completion_index = completions.size() - 1;
	    }
	    else {
		current_completion_index--;
	    }
	    root = completions[current_completion_index];
	    root_to_display = root;
	    return false;
	}
    }
    
    // tab,sIh,J}ȊÔȂ镶͂Ă⊮[h𔲂B
    completion_mode = false;
    
    if (SkkConfig::config().isToggleKanaKey(inputChar)) {
	if (status == STATUS_WHITE_ROOT || status == STATUS_WHITE_OKURI) {
	    // currenInputModeterminateāArootgetStringToFix()̖߂lǉB
	    parent->getCurrentInputMode().terminate();
	    CppCFString to_fix(parent->getCurrentInputMode().getStringToFix());
	    if (status == STATUS_WHITE_ROOT) {
		root += to_fix;
	    }
	    
	    // rootSpЉɕϊĂm肵ďIB
	    // SpЉ[h畽֕ϊB
	    if (parent->isZenKataInputMode()) {
		parent->fix(HiraganaInputMode::convert(root));
	    }
	    else {
		parent->fix(ZenKataInputMode::convert(root));
	    }
	    
	    parent->setEnabledKanjiMode(false);
	    initialize();
	    
	    return false;
	}
	else if (status == STATUS_BLACK || status == STATUS_BLACK_WITH_WINDOW) {
	    // ݂̌mB
	    // 1) SpЉ[hȂ畽[h
	    // 2) ȊOȂSpЉ[h
	    
	    parent->setEnabledKanjiMode(false);
	    if (parent->isZenKataInputMode()) {
		parent->goHiraganaInputMode();
	    }
	    else {
		parent->goZenKataInputMode();
	    }
	    return true;
	}
    }
    
    if (SkkConfig::config().isPrevKouhoKey(inputChar)) {
	if (status == STATUS_BLACK) {
	    // ǑցBŏ̌₾灤[h֖߂B
	    if (current_candidate_index > 0) {
		current_candidate_index--;
	    }
	    else {
		if (okuri.length() > 0)
		    status = STATUS_WHITE_OKURI;
		else
		    status = STATUS_WHITE_ROOT;
	    }
	    return false;
	}
	else if (status == STATUS_BLACK_WITH_WINDOW) {
	    // deleteꂽ̂ƌ
	    handleBackSpace();
	    return false;
	}
	// ȊOȂfʂ
    }
    
    if (SkkConfig::config().isPurgeFromJisyoKey(inputChar)) {
	if (status == STATUS_BLACK) {
	    // 폜vvgoB̕ϊ̓[U[ɍڂĂȂƂĂ
	    // ʓ|Ȃ̂ł̎킴킴mFȂBv]łΕʂB
	    status = STATUS_PROMPT;
	    prompt_type = PROMPT_DELETE_FROM_USER_DIC;
	    prompt_input.clear();
	    prompt.clear().append(0x672c).append(0x5f53).append(0x306b).append(' ');
	    if (okuri.length() > 0) {
		prompt.append('/').append(root).append('*').append(okuri).append('/');
	    }
	    else {
		prompt.append('/').append(root).append('/');
	    }
	    prompt.append(candidates[current_candidate_index]).append("/ ");
	    prompt.append(0x3092).append(0x524a).append(0x9664).append(0x3057).append(0x3066).
		append(0x3082).append(0x5b9c).append(0x3057).append(0x3044).append(0x3067).
		append(0x3059).append(0x304b).append(0xff1f).append("(yes/no)");
	    return false;
	}
    }
    
    if (SkkConfig::config().isToggleKatakanaKey(inputChar)) {
	if (status == STATUS_WHITE_ROOT || status == STATUS_WHITE_OKURI) {
	    // currenInputModeterminateāArootgetStringToFix()̖߂lǉB
	    parent->getCurrentInputMode().terminate();
	    CppCFString to_fix(parent->getCurrentInputMode().getStringToFix());
	    if (status == STATUS_WHITE_ROOT) {
		root += to_fix;
	    }
	    // root𔼊pЉɕϊĂm肵ďIB
	    // pЉ[h畽ɕϊB
	    if (parent->isHanKataInputMode()) {
		parent->fix(HiraganaInputMode::convert(root));
	    }
	    else {
		parent->fix(HanKataInputMode::convert(root));
	    }
	    
	    parent->setEnabledKanjiMode(false);
	    initialize();
	    return false;
	}
	else if (status == STATUS_BLACK || status == STATUS_BLACK_WITH_WINDOW) {
	    // ݂̌m肵ĂA
	    // 1) pЉ[hȂ畽[h
	    // 2) ȊOȂ甼pЉ[h
	    
	    parent->setEnabledKanjiMode(false);
	    if (parent->isHanKataInputMode()) {
		parent->goHiraganaInputMode();
	    }
	    else {
		parent->goHanKataInputMode();
	    }
	    return true;
	}
    }
    
    if (SkkConfig::config().isKakuteiKey(inputChar)) {
	return true; // m̂
    }
    
    if (SkkConfig::config().isLatinModeKey(inputChar)
	|| SkkConfig::config().isJisx0208LatinModeKey(inputChar)) {
	if (status == STATUS_WHITE_ROOT || status == STATUS_WHITE_OKURI || status == STATUS_BLACK) {
	    // ꂼm肵Ĕp/Spp[hցB
	    if (SkkConfig::config().isLatinModeKey(inputChar)) {
		parent->goHanAscInputMode();
	    }
	    else {
		parent->goZenAscInputMode();
	    }
	    return true;
	}
	// IʂȂfʂB
    }
    
    if (SkkConfig::config().isAbbrevModeKey(inputChar)) {
	// ݂̌m肵AďI邪A
	// ɍ̕\/fixfȂƍ̂
	// ϊ[hIĂparenthandleInputɓnB
	// KanjiConversionModeSTATUS_NULL̎ɂ
	// kIgnoreThisԂB
	
	parent->fix(this->getStringToFix());
	parent->handleInput(inputChar);
	return false;
    }
    
    if (SkkConfig::config().isNextKouhoKey(inputChar)) {
	if (status == STATUS_WHITE_ROOT || status == STATUS_WHITE_OKURI) {
	    // currenInputModeterminateāArootokurigetStringToFix()̖߂lǉB
	    parent->getCurrentInputMode().terminate();
	    if (status == STATUS_WHITE_ROOT) {
		root += parent->getCurrentInputMode().getStringToFix();
		root_to_display.clear().append(root).append(parent->getCurrentInputMode().getStringToDisplay());
		
		if (root.length() == 0) {
		    // I
		    parent->setEnabledKanjiMode(false);
		    parent->display(CppCFString());
		    initialize();
		    return false;
		}
		
		askServerTheCandidates(false);
	    }
	    else {
		okuri += parent->getCurrentInputMode().getStringToFix();
		okuri_to_display.clear().append(okuri).append(parent->getCurrentInputMode().getStringToDisplay());
		
		askServerTheCandidates(true);
	    }
	    
	    if (candidates.size() > 0) {
		// ₪݂B[hցB
		status = STATUS_BLACK;
		current_candidate_index = 0; // ŏ̌B
	    }
	    else {
		// ₪BPo^JnB
		startRegisteringWord();
	    }
	}
	else if (status == STATUS_BLACK) {
	    // ̌
	    current_candidate_index++;
	    
	    if (current_candidate_index >= candidates.size()) { // I܂ōsPo^JnB
		startRegisteringWord();
	    }
	    else if (current_candidate_index >= show_cands_window_after_Nth_cand - 1) {
		// ȍ~̃[e[VŌIEChE\B
		status = STATUS_BLACK_WITH_WINDOW;
		openCandidatesWindow();
	    }
	}
	else if (status == STATUS_BLACK_WITH_WINDOW) {
	    if (candidates_window_current_frame < candidates_window_num_of_frames - 1) {
		// Ō̃t[łȂ΁Ãt[֍sB
		goToNextCandidatesFrame();
	    }
	    else {
		// Ō̃t[APo^JnB
		closeCandidatesWindow();
		startRegisteringWord();
	    }
	}
	return false;
    }
    
    // ϊ|CgL[ꂽ̓
    if (SkkConfig::config().isSetHenkanPointKey(inputChar)) {
	if (status == STATUS_NULL ||
	    status == STATUS_WHITE_ROOT && root.length() == 0) {
	    status = STATUS_WHITE_ROOT; // [hցB
	    
	    // ʕ\p
	    root_to_display.clear().append(root).append(parent->getCurrentInputMode().getStringToDisplay());
	}
	return false;
    }
    
    if (inputChar >= 'A' && inputChar <= 'Z') {
	if (status == STATUS_NULL ||
	    status == STATUS_WHITE_ROOT && root.length() == 0) {
	    status = STATUS_WHITE_ROOT; // [hցB
	    
	    // ɂĉ͂sB
	    parent->getCurrentInputMode().handleInput(inputChar + 0x20);
	    
	    // ̓[hfixobt@root֒ǉB
	    root += parent->getCurrentInputMode().getStringToFix();
	    
	    // ʕ\p
	    root_to_display.clear().append(root).append(parent->getCurrentInputMode().getStringToDisplay());
	    
	    // okuri͏ĂB̏ԂŊm肳ꂽׂ̈ɁB
	    okuri.clear();
	}
	else if (status == STATUS_WHITE_ROOT) {
	    // terminateOɍ̓͂ށB
	    // ȂKesSiteuāvɂȂĂ܂B
	    // AA̓͂̏Aꕶŏo͂𐶐ꍇɂ
	    // (WIɂ͕ꉹn)ȂȂB
	    CppCFString tmp_queue(inputChar + 0x20);
	    CppCFString result =
		KanaTreeController::defaultTreeController().trace(
		    KanaTreeController::OUTPUT_HIRAGANA,
		    tmp_queue);
	    if (result.length() == 0 && tmp_queue != "n") {
		parent->getCurrentInputMode().handleInput(inputChar + 0x20); // 
	    }
	    
	    // currentInputModeterminateāArootgetStringToFix()̖߂lǉB
	    parent->getCurrentInputMode().terminate();
	    root += parent->getCurrentInputMode().getStringToFix();
	    root_to_display.clear().append(root).append(parent->getCurrentInputMode().getStringToDisplay());
	    
	    status = STATUS_WHITE_OKURI; // 艼[hցB
	    
	    okuri_head = inputChar + 0x20; // 
	    
	    // ɂĉ͂sB
	    bool fix = parent->getCurrentInputMode().handleInput(okuri_head);
	    
	    // ̓[hfixobt@okuriփRs[B
	    okuri += parent->getCurrentInputMode().getStringToFix();
	    
	    // ʕ\p
	    okuri_to_display.clear().append(okuri).append(parent->getCurrentInputMode().getStringToDisplay());
	    
	    // L[łȂ΂܂ϊĂ܂ȂB
	    if (fix && parent->getCurrentInputMode().isQueueEmpty()) {
		// ϊB
		return handleInput(' '); // ē˓BӖIɂ͍ċAł͂ȂcontinueB
	    }
	}
	else if (status == STATUS_WHITE_OKURI) {
	    // ɂĉ͂sB
	    bool fix = parent->getCurrentInputMode().handleInput(inputChar + 0x20);
	    
	    // ̓[hfixobt@okuriփRs[B
	    okuri += parent->getCurrentInputMode().getStringToFix();
	    
	    // ʕ\p
	    okuri_to_display.clear().append(okuri).append(parent->getCurrentInputMode().getStringToDisplay());
	    
	    // L[łȂ΂܂ϊĂ܂ȂB
	    if (fix && parent->getCurrentInputMode().isQueueEmpty()) {
		// ϊB
		return handleInput(' '); // ē˓BӖIɂ͍ċAł͂ȂcontinueB
	    }
	}
	else if (status == STATUS_BLACK) {
	    // ݂̌m肵AUĂē˓B
	    // E͂ȂdȂB
	    
	    // IɑM
	    sendCurrentCandidateToServerToRegister();
	    
	    // m
	    parent->fix(candidates[current_candidate_index]);
	    
	    initialize();
	    return handleInput(inputChar); // ē˓
	}
	else if (status == STATUS_BLACK_WITH_WINDOW) {
	    // ASDFJKLi1t[̌␔7菭Ȃ΂Bj̕ł
	    // ̌m肷B
	    // łȂΌ݂̌m肷B
	    int cand_index = CppCFString("ASDFJKL").substring(0,candidates_window_cands_per_frame).indexOf(inputChar);
	    if (cand_index != -1) {
		unsigned new_index = current_candidate_index + cand_index;
		if (new_index < candidates.size()) {
		    // ̃CfBbNXȂmB
		    current_candidate_index = new_index;
		    return true;
		}				
	    }
	    return false;
	}
	else if (status == STATUS_PROMPT) {
	    // ̂܂prompt_inputɒǉB
	    prompt_input += inputChar;
	}
    }
    else {
	if (status == STATUS_WHITE_ROOT) {
	    // ͂sB
	    parent->getCurrentInputMode().handleInput(inputChar);
	    
	    // rootɒǉB
	    root += parent->getCurrentInputMode().getStringToFix();
	    
	    // ʕ\p
	    root_to_display.clear().append(root).append(parent->getCurrentInputMode().getStringToDisplay());
	}
	else if (status == STATUS_WHITE_OKURI) {
	    // currentInputMode::handleInput()sAtrueԂ炻𑗂艼ƂĕϊB
	    // łȂ΁AcurrentInputMode::handleInput()ɓnB
	    bool fix = parent->getCurrentInputMode().handleInput(inputChar);
	    const CppCFString& kanainput_result = parent->getCurrentInputMode().getStringToFix();
	    
	    // okuriɒǉB
	    okuri += kanainput_result;
	    
	    // ʕ\p
	    okuri_to_display.clear().append(okuri).append(parent->getCurrentInputMode().getStringToDisplay());
	    
	    // L[łȂ΂܂ϊȂB
	    if (fix && parent->getCurrentInputMode().isQueueEmpty()) {
		// ϊB
		return handleInput(' '); // ē˓BӖIɂ͍ċAł͂ȂcontinueB
	    }
	}
	else if (status == STATUS_BLACK) {
	    // ݂̌m肵AďI邪A
	    // ɍ̕\/fixfȂƍ̂
	    // ϊ[hIĂparenthandleInputɓnB
	    // KanjiConversionModeSTATUS_NULL̎ɂ
	    // kIgnoreThisԂB
	    
	    // IɑM
	    sendCurrentCandidateToServerToRegister();
	    
	    // ȂAIɊm肳B
	    parent->fix(candidates[current_candidate_index]);
	    
	    parent->setEnabledKanjiMode(false);
	    initialize();
	    
	    parent->handleInput(inputChar);
	}
	else if (status == STATUS_BLACK_WITH_WINDOW) {
	    // asdfjkli1t[̌␔7菭Ȃ΂Bj̕ł
	    // ̌m肷B
	    // łȂΌ݂̌m肷B
	    int cand_index = CppCFString("asdfjkl").substring(0,candidates_window_cands_per_frame).indexOf(inputChar);
	    if (cand_index != -1) {
		unsigned new_index = current_candidate_index + cand_index;
		if (new_index < candidates.size()) {
		    // ̃CfBbNXȂmB
		    current_candidate_index = new_index;
		    return true;
		}				
	    }
	    return false;
	}
	else if (status == STATUS_PROMPT) {
	    // prompt_inputɂ̂܂܉B
	    prompt_input += inputChar;
	}
    }
    return false;
}

void KanjiConversionMode::startRegisteringWord() {
    CppCFString register_title;
    if (okuri.length() > 0) {
	register_title.append(root).append('*').append(okuri);
    }
    else {
	register_title = root;
    }
    register_title = HiraganaInputMode::convert(register_title); // Љ͕ɕϊ
	
    word_register_mode = new WordRegisterMode(*this,register_title);
    word_register_mode->redisplay();
    word_register_mode->updatePencilMenuIcon();
}

bool KanjiConversionMode::handleBackSpace() {
    if (word_register_mode) {
	word_register_mode->handleBackSpace();
	return true; // WordRegisterMode::handleBackSpace()ԂƂ{͏ȂB
    }
    
    if (status == STATUS_NULL) {
	return false;
    }
    
    if (status == STATUS_WHITE_ROOT) {
	// hiraganaInputMode->handleBackSpace()trueԂroot_to_displayXVB
	// falseԂ
	//		rootȂ珉ďIB
	//		łȂroot̍Ō̕B
	if (parent->getCurrentInputMode().handleBackSpace()) {
	    root_to_display.clear().append(root).append(parent->getCurrentInputMode().getStringToDisplay());
	}
	else {
	    if (root.length() == 0) {
		parent->setEnabledKanjiMode(false);
		parent->display(CppCFString());
		initialize();
	    }
	    else {
		completion_mode = false; // ⊮[hꍇ͕⊮[h𔲂B
		root.eraseLast(1);
		root_to_display = root;
	    }
	}
    }
    else if (status == STATUS_WHITE_OKURI) {
	// hiraganaInputMode->handleBackSpace()
	// trueԂ
	//      okuri_to_displayXVB
	//      ̎AokuriꕶSTATUS_WHITE_ROOT֖߂B
	// falseԂ
	//		okuriꕶSTATUS_WHITE_ROOT֖߂B
	//		񕶎ȏ゠okuri̍Ō̕B
	if (parent->getCurrentInputMode().handleBackSpace()) {
	    okuri_to_display.clear().append(okuri).append(parent->getCurrentInputMode().getStringToDisplay());
	    if (okuri.length() <= 1) {
		okuri.clear();
		status = STATUS_WHITE_ROOT;
	    }
	}
	else {
	    if (okuri.length() <= 1) {
		okuri.clear();
		status = STATUS_WHITE_ROOT;
	    }
	    else {
		okuri.eraseLast(1);
		okuri_to_display = okuri;
	    }
	}
    }
    else if (status == STATUS_BLACK) {
	// ݂̌m肵ĂŌ̕폜B
	// ďďIB
	CppCFString str_to_fix(candidates[current_candidate_index]);
	str_to_fix.eraseLast(1);
	
	// IɑM
	sendCurrentCandidateToServerToRegister();
	
	// ̎
	parent->fix(str_to_fix);
	parent->display(CppCFString());
	
	parent->setEnabledKanjiMode(false);
	initialize();
    }
    else if (status == STATUS_BLACK_WITH_WINDOW) {
	// ŏ̃t[Ȃ灥[h֖߂B
	// łȂΑÕt[ցB
	if (candidates_window_current_frame == 0) {
	    // EChEJOɖ߂
	    current_candidate_index = show_cands_window_after_Nth_cand - 2;
	    
	    // EChEB
	    closeCandidatesWindow();
	    
	    status = STATUS_BLACK;
	}
	else {
	    goToPrevCandidatesFrame();
	}
    }
    else if (status == STATUS_PROMPT) {
	// prompt_inputłȂ΍Ō̕B
	if (prompt_input.length() > 0) {
	    prompt_input.eraseLast(1);
	}
    }
    return true;
}

bool KanjiConversionMode::handleReturn() {
    if (word_register_mode) {
	word_register_mode->handleReturn();
	
	if (word_register_mode) {
	    // handleReturnwordRegistrationFinishedĂ΂ꂽword_register_mode͔j邽߁B
	    word_register_mode->redisplay();
	}
	
	return true; // trueԂB
    }
    
    if (status == STATUS_PROMPT) {
	switch (prompt_type) {
	case PROMPT_DELETE_FROM_USER_DIC: {
	    // prompt_inputyesȂ폜STATUS_WHITE_ROOTɖ߂B
	    // ȊOȂSTATUS_BLACKɖ߂B
	    if (prompt_input == "yes") {
		// 폜
		makeServerRemoveWord();
		
		// STATUS_WHITE_ROOT
		if (okuri.length() > 0) {
		    root += okuri;
		    root_to_display = root;
		    
		    okuri.clear();
		    okuri_to_display.clear();
		}
		status = STATUS_WHITE_ROOT;
	    }
	    else {
		status = STATUS_BLACK;
	    }
	}
	    break;
	    
	default:
	    break;
	}
	return true;
    }
    
    if (ClientConfiguration::sharedConfiguration().isSkkEggLikeNewline()) {
	// skk-egglike-newline[hȂA͂Ŋm肳Ă珉trueԂB
	
	// ȂAIɊm肳B
	parent->fix(getStringToFix());
	return true;
    }
    else {
	return false;
    }
}

bool KanjiConversionMode::handleArrow(int kind) {
    if (word_register_mode) {
	return word_register_mode->handleArrow(kind); // Ϗ
    }
    
    // Cvbg[hhandleArrowĂсAgetStringToFix()ĂԂAĂԂB
    parent->getCurrentInputMode().handleArrow(kind);
    parent->getCurrentInputMode().getStringToFix();
    
    if (status == STATUS_BLACK || status == STATUS_BLACK_WITH_WINDOW) {
	// IɑM
	sendCurrentCandidateToServerToRegister();
	
	// ݂̌mB
	parent->fix(candidates[current_candidate_index]);
	
	if (status == STATUS_BLACK_WITH_WINDOW) {
	    // EChEB
	    closeCandidatesWindow();
	}
    }
    
    // ϊ[hIB
    parent->setEnabledKanjiMode(false);
    initialize();
    return false;
}

bool KanjiConversionMode::handleCg() {
    if (word_register_mode) {
	return word_register_mode->handleCg();
    }
    
    // Cvbg[hhandleCgĂсAgetStringToFix()ĂԂAĂԂB
    parent->getCurrentInputMode().handleCg();
    parent->getCurrentInputMode().getStringToFix();
    
    if (status == STATUS_WHITE_ROOT || status == STATUS_WHITE_OKURI) {
	// ϊ[hI
	parent->setEnabledKanjiMode(false);
	initialize();
    }
    else if (status == STATUS_BLACK || status == STATUS_BLACK_WITH_WINDOW) {
	// rootokuriSTATUS_WHITE_ROOT
	// EChEJĂB
	if (status == STATUS_BLACK_WITH_WINDOW) {
	    closeCandidatesWindow();
	}
	
	root += okuri;
	okuri.clear();
	
	root_to_display = root;
	okuri_to_display.clear();
	
	status = STATUS_WHITE_ROOT;
    }
    return true;
}

bool KanjiConversionMode::handleCy() {
    if (word_register_mode) {
	return word_register_mode->handleCy();
    }
    else {
	return false;
    }
}

bool KanjiConversionMode::handleTab() {
    if (word_register_mode) {
	return word_register_mode->handleTab();
    }
    
    if (status == STATUS_WHITE_ROOT) {
	if (completion_mode) {
	    // ̌ցBŌ܂ōsŏɖ߂B
	    current_completion_index++;
	    if (current_completion_index >= completions.size()) {
		current_completion_index = 0;
	    }
	    root = completions[current_completion_index];
	    root_to_display = root;
	}
	else {
	    // currentInputModeterminateāArootgetStringToFix()̖߂lǉB
	    parent->getCurrentInputMode().terminate();
	    root += parent->getCurrentInputMode().getStringToFix();
	    root_to_display.clear().append(root).append(parent->getCurrentInputMode().getStringToDisplay());
	    
	    // Iɖ₢킹B
	    askServerTheCompletions();
	    
	    // ⊮̌₪Ε⊮JnBΉɖ߂B
	    if (completions.size() > 0) {
		completion_mode = true;
		root = completions[0];
		root_to_display = root;
	    }
	}
	return true;
    }
    
    return false;
}

void KanjiConversionMode::terminate() {
    D_PRINTF("KanjiConversionMode::terminate\n");
    
    if (word_register_mode) {
	word_register_mode->terminate();
	delete word_register_mode;
	word_register_mode = NULL;
    }

    // currenInputModeterminateāArootgetStringToFix()̖߂lǉB
    parent->getCurrentInputMode().terminate();
    CppCFString to_fix(parent->getCurrentInputMode().getStringToFix());
    if (status == STATUS_WHITE_ROOT) {
	root += to_fix;
    }
    
    parent->fix(this->getStringToFix()); // m肷
    parent->setEnabledKanjiMode(false);
    initialize();
}

void KanjiConversionMode::wordRegistrationFinished() {
    if (word_register_mode) {
	CppCFString content = word_register_mode->getContent();
	if (content.length() > 0) {
	    // IɑMB
	    if (okuri.length() > 0) {
		if (SkkConfig::config().useKanaToOkuri()) {
#ifdef DEBUG
		    cerr << "(A)OKURI-HEAD: " << okuri[0] << " - " << SkkConfig::config().kanaToOkuri(okuri[0]) << endl;
#endif
		    sendWordToServerToRegister(root + SkkConfig::config().kanaToOkuri(okuri[0]), okuri, content);
		} else {
		    sendWordToServerToRegister(root + okuri_head, okuri, content);
		}
	    }
	    else
		sendWordToServerToRegister(root, content);
	    
	    // mB
	    parent->fix(content + okuri); // 艼t
	    
	    // IB
	    parent->setEnabledKanjiMode(false);
	    initialize();
	}
	else {
	    if (okuri.length() > 0) { // 艼
		status = STATUS_WHITE_OKURI;
	    }
	    else {
		status = STATUS_WHITE_ROOT;
	    }
	}
	
	delete word_register_mode;
	word_register_mode = NULL;
	parent->updatePencilMenuIcon();
    }
}

void KanjiConversionMode::wordRegistrationCanceled() {
    delete word_register_mode;
    word_register_mode = NULL;
    parent->updatePencilMenuIcon();
    
    if (status == STATUS_WHITE_ROOT || status == STATUS_WHITE_OKURI) {
	// rootokuriSTATUS_WHITE_ROOT
	root += okuri;
	okuri.clear();
	
	root_to_display = root;
	okuri_to_display.clear();
	
	status = STATUS_WHITE_ROOT;
    }
    else if (status == STATUS_BLACK) {
	current_candidate_index = candidates.size() - 1; // Ō̌ցB
    }
    else if (status == STATUS_BLACK_WITH_WINDOW) {
	// EChEJčŌ̃t[o
	current_candidate_index = show_cands_window_after_Nth_cand - 1; // goToNextCandidatesFrame()łB
	openCandidatesWindow();
	for (unsigned i = 1;i < candidates_window_num_of_frames;i++) {
	    goToNextCandidatesFrame();
	}
    }
}

CppCFString KanjiConversionMode::getStringToDisplay() {
    if (word_register_mode) {
	// BȂƐ܊pWordRegisterModeݒ肵ׂĂ܂B
	return CppCFString().append(kIgnoreThis);
    }
    
    if (status == STATUS_WHITE_ROOT) {
	// root
	return CppCFString().append(0x25bd).append(root_to_display);
    }
    else if (status == STATUS_WHITE_OKURI) {
	// root*okuri
	return CppCFString().append(0x25bd).append(root_to_display).append('*').append(okuri_to_display);
    }
    else if (status == STATUS_BLACK || status == STATUS_BLACK_WITH_WINDOW) {
	// ݂̌
	return CppCFString().append(0x25bc).append(candidates[current_candidate_index]);
    }
    else if (status == STATUS_PROMPT) {
	// [vvg][]
	return CppCFString('[').append(prompt).append("][").append(prompt_input).append(kCaret).append(']');
    }
    else {
	return CppCFString().append(kIgnoreThis);
    }
}

CppCFString KanjiConversionMode::getStringToFix() {
    CppCFString result;
    if (status == STATUS_WHITE_ROOT) {
	result = root;
    }
    else if (status == STATUS_WHITE_OKURI) {
	result = root + okuri;
    }
    else if (status == STATUS_BLACK) {
	result = candidates[current_candidate_index];
	
	// IɑM
	sendCurrentCandidateToServerToRegister();
    }
    else if (status == STATUS_BLACK_WITH_WINDOW) {
	result = candidates[current_candidate_index];
	
	// IɑM
	sendCurrentCandidateToServerToRegister();
	
	// EChEB
	closeCandidatesWindow();
    }
    else /* if (status == STATUS_NULL) */ {
	//result = str_return_as_fix_when_status_null;
	result = CppCFString().append(kIgnoreThis);
    }
	
    // ꂪĂ΂ꂽƂƂfixƂƂȂ̂Ŋ[hIB
    // B
    parent->setEnabledKanjiMode(false);
    initialize();
    
    return result;
}

void KanjiConversionMode::initialize() {
    status = STATUS_NULL;
    root.clear();
    okuri.clear();
    root_to_display.clear();
    okuri_to_display.clear();
    candidates.clear();
    current_candidate_index = 0;
    completion_mode = false;
    completions.clear();
    current_completion_index = 0;
}

void KanjiConversionMode::sendCurrentCandidateToServerToRegister() {
    // IɑMB
    // Љ͕ɕϊB
    if (okuri.length() > 0) {
	if (SkkConfig::config().useKanaToOkuri()) {
#ifdef DEBUG
	    cerr << "(B)OKURI-HEAD: " << okuri[0] << " - " << SkkConfig::config().kanaToOkuri(okuri[0]) << endl;
#endif
	    sendWordToServerToRegister(
		HiraganaInputMode::convert(root) + SkkConfig::config().kanaToOkuri(okuri[0]), 
		HiraganaInputMode::convert(okuri),
		candidates[current_candidate_index].clone().eraseLast(okuri.length())); // 艼B
	} else {
	    sendWordToServerToRegister(
		HiraganaInputMode::convert(root) + okuri_head,
		HiraganaInputMode::convert(okuri),
		candidates[current_candidate_index].clone().eraseLast(okuri.length())); // 艼B
	}
    }
    else {
	sendWordToServerToRegister(
	    HiraganaInputMode::convert(root),
	    candidates[current_candidate_index]);
    }
}

void KanjiConversionMode::askServerTheCandidates(bool hasOkuri) {
    // 艼ΐ擪+A-tB
    CppCFString query;
    if (hasOkuri) {
	if (SkkConfig::config().useKanaToOkuri()) {
#ifdef DEBUG		
	    cerr << "(C)OKURI-HEAD: " << okuri[0] << " - " << SkkConfig::config().kanaToOkuri(okuri[0]) << endl;
#endif
	    query = CppCFString().append('+').append(root).append(SkkConfig::config().kanaToOkuri(okuri[0])).append(' ').append(okuri);
	} else {
	    // @@+r 
	    query = CppCFString().append('+').append(root).append(okuri_head).append(' ').append(okuri);
	}
    }
    else {
	// ȁ@@-
	query = CppCFString().append('-').append(root);
    }
    
    // SpƔp𕽉ɕϊ
    query = HiraganaInputMode::convert(query);
    
    D_PRINTF("query: %s\n",query.toCString(kCFStringEncodingEUC_JP));
    
    // Iɐq˂
    CppCFData cfdata_query(query.toCFData());
    current_candidate_index = 0;
    ::askServerTheCandidates(cfdata_query,candidates);
}

// 񃁃o
void askServerTheCandidates(const CppCFData& query,vector<CppCFString>& candidates) {
    static CppCFString space(" ");
    static CppCFString nbsp("[20]");
    
    CppCFString reply(
	ServerConnectionFactory::sharedFactory().newConnection().send(kSKKMessageSearch,query).getData());
    
    // 󔒂ŋ؂B
    candidates.clear();
    const int reply_len = reply.length();
    int blockstart = 0;
    for (int i = 0;i < reply_len;i++) {
	if (reply[i] == ' ') {
	    if (i > blockstart) {
		// ǉ
		addCandidate(
		    reply.substring(blockstart,i).replace(nbsp,space),candidates);
	    }
	    blockstart = i+1;
	}
    }
    if (reply_len > blockstart) {
	// ǉ
	addCandidate(
	    reply.substring(blockstart,reply_len).replace(nbsp,space),candidates);
    }
}
void addCandidate(const CppCFString& candidate,vector<CppCFString>& candidates) {
    static CppCFString blockopen("[");
    static CppCFString blockopen_ref("[5b]");
    static CppCFString slash("/");
    static CppCFString slash_ref("[2f]");
    static CppCFString semicolon(";");
    static CppCFString semicolon_ref("[3b]");
    
    // _ł͒߂𖳏ō폜B
    const int semicolon_pos = candidate.indexOf(';');
    CppCFString cand;
    if (semicolon_pos != -1) {
	cand = candidate.substring(0,semicolon_pos);
    }
    else {
	cand = candidate;
    }
    
    candidates.push_back(
	cand.replace(slash_ref,slash).replace(semicolon_ref,semicolon).replace(blockopen_ref,blockopen));
}

void KanjiConversionMode::makeServerRemoveWord() {
    CppCFString query;
    if (okuri.length() > 0) {
	//  @@+r  
	// ǂ݂ɂĂ͑SpƔp𕽉ɕϊ
	query.append('+');
	if (SkkConfig::config().useKanaToOkuri()) {
#ifdef DEBUG
	    cerr << "(D)OKURI-HEAD: " << okuri[0] << " - " << SkkConfig::config().kanaToOkuri(okuri[0]) << endl;
#endif
	    query.append(HiraganaInputMode::convert(root)).append(SkkConfig::config().kanaToOkuri(okuri[0]));
	} else {
	    query.append(HiraganaInputMode::convert(root)).append(okuri_head);
	}
	query.append(' ').append(HiraganaInputMode::convert(okuri));
	query.append(' ').append(candidates[current_candidate_index].clone().eraseLast(okuri.length())); // 艼B
    }
    else {
	//  @@- 
	// ǂ݂ɂĂ͑SpƔp𕽉ɕϊ
	query.append('-').append(HiraganaInputMode::convert(root)).append(' ');
	query.append(candidates[current_candidate_index]);
    }
    
    // Iɓn
    CppCFData cfdata_query(query.toCFData());
    ::makeServerRemoveWord(cfdata_query);
}

// 񃁃o
void makeServerRemoveWord(const CppCFData& query) {
    ServerConnectionFactory::sharedFactory().newConnection().send(kSKKRemoveThisFromUserDic,query);
}

void KanjiConversionMode::askServerTheCompletions() {
    // SpƔp𕽉ɕϊ
    CppCFString query = HiraganaInputMode::convert(root);
    
    // Iɐq˂
    CppCFData cfdata_query(query.toCFData());
    current_completion_index = 0;
    ::askServerTheCompletions(cfdata_query,completions);
}

void askServerTheCompletions(const CppCFData& query,vector<CppCFString>& completions) {
    CppCFString reply(
	ServerConnectionFactory::sharedFactory().newConnection().send(kSKKFetchCompletions,query).getData());
    
    CppCFString space(" ");
    CppCFString nbsp("[20]");
    
    // 󔒂ŋ؂B
    completions.clear();
    const int reply_len = reply.length();
    int blockstart = 0;
    for (int i = 0;i < reply_len;i++) {
	if (reply[i] == ' ') {
	    if (i > blockstart) {
		// ǉ
		completions.push_back(reply.substring(blockstart,i).replace(nbsp,space));
	    }
	    blockstart = i+1;
	}
    }
    if (reply_len > blockstart) {
	// ǉ
	completions.push_back(reply.substring(blockstart,reply_len).replace(nbsp,space));
    }
}

// 񃁃o
void sendWordToServerToRegister(const CppCFString& index,const CppCFString& okuri,const CppCFString& kanji) {
    CppCFString space(" ");
    CppCFString nbsp("[20]");
    
    CppCFData query(CppCFString().append('+').append(index.replaceClone(space,nbsp)).
		    append(' ').append(okuri.replaceClone(space,nbsp)).
		    append(' ').append(kanji.replaceClone(space,nbsp)).toCFData());
    
    ServerConnectionFactory::sharedFactory().newConnection().send(kSKKRegisterThisToUserDic,query);
}

void sendWordToServerToRegister(const CppCFString& index,const CppCFString& kanji) {
    static CppCFString space(" ");
    static CppCFString nbsp("[20]");
    static CppCFString blockopen("[");
    static CppCFString blockopen_ref("[5b]");
    static CppCFString slash("/");
    static CppCFString slash_ref("[2f]");
    static CppCFString semicolon(";");
    static CppCFString semicolon_ref("[3b]");
    
    CppCFString encoded_kanji = 
	kanji.replaceClone(blockopen,blockopen_ref).replace(slash,slash_ref).
	replace(semicolon,semicolon_ref).replace(space,nbsp);
    
    CppCFData query(CppCFString().append('-').append(index.replaceClone(space,nbsp)).
		    append(' ').append(encoded_kanji).toCFData());
    
    ServerConnectionFactory::sharedFactory().newConnection().send(kSKKRegisterThisToUserDic,query);
}

void KanjiConversionMode::openCandidatesWindow() {
    // candidates4Ԗڈȍ~̍ڂ瑗艼菜zB
    // 艼Ήꂸ̂܂܃Rs[B
    CppCFString space(" ");
    CppCFString nbsp("[20]");
    
    vector<CppCFString> cands_without_okuri;
    unsigned okuri_length = okuri.length();
    if (okuri_length > 0) {
	for (vector<CppCFString>::const_iterator ite = candidates.begin() + (show_cands_window_after_Nth_cand-1);ite != candidates.end();ite++) {
	    cands_without_okuri.push_back(
		ite->substring(0,ite->length() - okuri_length).replace(space,nbsp));
	}
    }
    else {
	for (vector<CppCFString>::const_iterator ite = candidates.begin() + (show_cands_window_after_Nth_cand-1);ite != candidates.end();ite++) {
	    cands_without_okuri.push_back(ite->replaceClone(space,nbsp));
	} 
	//cands_without_okuri.assign(candidates.begin()+(show_cands_window_after_Nth_cand-1),candidates.end());
    }
    
    ::openCandidatesWindow(join(' ',cands_without_okuri),
			   candidates_window_id,
			   candidates_window_cands_per_frame,
			   candidates_window_num_of_frames);
    candidates_window_current_frame = 0;
}

// 񃁃o
void openCandidatesWindow(const CppCFString& str_candidates,unsigned& window_id,unsigned& cands_per_frame,unsigned& num_of_frames) {
    // CC͂sĂʒu擾B
    struct OffsetToPosParams offset_to_pos_params;
    offset_to_pos_params.fRefCon = 0;
    offset_to_pos_params.fTextOffset = 0;
    offset_to_pos_params.fSLRec.fScript = smJapanese;
    offset_to_pos_params.fSLRec.fLanguage = langJapanese;
    offset_to_pos_params.fLeadingEdge = FALSE;
    BIMHandleOffsetToPos(&offset_to_pos_params);
    SInt16 qd_x = offset_to_pos_params.fReplyPoint.h;
    SInt16 qd_y = offset_to_pos_params.fReplyPoint.v;
    
    // T[o[ɑtCFData쐬B
    CppCFData attachment;
    attachment.append(qd_x).append(qd_y).append(str_candidates.toCFData());
    
    // M
    CppCFData reply(
	ServerConnectionFactory::sharedFactory().newConnection().send(
	    kSKKCreateCandidatesWindow,attachment));
    
    // ԂĂf[^𕪉ĕۑB
    // +0  IID             2oCg
    // +2  1t[ɕ\̐  2oCg
    // +4  t[                 2oCg
    //CppCFData reply(reply_data);
    window_id = reply.getUInt16(0);
    cands_per_frame = reply.getUInt16(2);
    num_of_frames = reply.getUInt16(4);
}

// 񃁃o
CppCFData newCFDataRefWithWindowID(unsigned window_id) {
    CppCFData result;
    result.append(static_cast<UInt16>(window_id));
    
    return result;
}

void KanjiConversionMode::goToNextCandidatesFrame() {
    // EChEID(UInt16)܂CFData쐬AI֑B
    ServerConnectionFactory::sharedFactory().newConnection().send(kSKKCandidatesWindowNext,newCFDataRefWithWindowID(candidates_window_id));
    
    // ݂̃t[ԍCNg
    candidates_window_current_frame++;
    
    // ݂̌ԍt[̌␔₷B
    current_candidate_index += candidates_window_cands_per_frame;
}

void KanjiConversionMode::goToPrevCandidatesFrame() {
    // EChEID(UInt16)܂CFData쐬AI֑B	
    ServerConnectionFactory::sharedFactory().newConnection().send(kSKKCandidatesWindowPrev,newCFDataRefWithWindowID(candidates_window_id));
    
    // ݂̃t[ԍfNg
    candidates_window_current_frame--;
    
    // ݂̌ԍt[̌␔炷B
    current_candidate_index -= candidates_window_cands_per_frame;
}

void KanjiConversionMode::closeCandidatesWindow() {
    // EChEID(UInt16)܂CFData쐬AI֑B
    ServerConnectionFactory::sharedFactory().newConnection().send(kSKKCloseCandidatesWindow,newCFDataRefWithWindowID(candidates_window_id));
}
