/*
	$Id: OkuriganaEntry.cpp,v 1.6 2002/11/29 12:23:11 phonohawk Exp $
	---------
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "OkuriganaEntry.h"

OkuriganaEntry::OkuriganaEntry()
{

}

OkuriganaEntry::OkuriganaEntry(const CppCFString& okurigana)
	:kana(okurigana)
{

}

OkuriganaEntry::OkuriganaEntry(const OkuriganaEntry& okurigana_entry)
{
	kana = okurigana_entry.getKana();
	kanji_candidates = okurigana_entry.getCandidates();
}

OkuriganaEntry::~OkuriganaEntry()
{
	
}

void OkuriganaEntry::add(const CppCFString& candidate)
{
	kanji_candidates.push_back(candidate);
}

void OkuriganaEntry::insert(const CppCFString& candidate,unsigned pos)
{
	kanji_candidates.insert(kanji_candidates.begin() + pos,candidate);
}

void OkuriganaEntry::remove(unsigned pos)
{
	kanji_candidates.erase(kanji_candidates.begin()+pos);
}

void OkuriganaEntry::removeRedundantItems()
{
	int size = kanji_candidates.size();
	for (int i = 0;i < size;i++)
	{
		CppCFString& cand_i = kanji_candidates[i];
		
		bool is_redundant = false;
		for (int j = 0;j < i;j++)
		{
			if (cand_i == kanji_candidates[j])
			{
				is_redundant = true;
				break;
			}
		}
		
		if (is_redundant)
		{
			kanji_candidates.erase(kanji_candidates.begin()+i);
			i--; // ̂ŁǍɑڂB
			size--; // ̂ŁB
		}
	}
}

const CppCFString& OkuriganaEntry::getKana() const
{
	return kana;
}

bool OkuriganaEntry::isWild() const
{
	return kana.length() == 0;
}

void OkuriganaEntry::setKana(const CppCFString& new_kana)
{
	kana = new_kana;
}

const vector<CppCFString>& OkuriganaEntry::getCandidates() const
{
	return kanji_candidates;
}

unsigned OkuriganaEntry::size() const
{
	return kanji_candidates.size();
}

const CppCFString& OkuriganaEntry::operator [](unsigned n) const
{
	return kanji_candidates[n];
}