/*  $Id: ParentInputMode.h,v 1.15 2004/01/21 08:34:09 phonohawk Exp $
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002-2004 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#pragma once
#include <Carbon/Carbon.h>
#include "CppCFString.h"
#include "InputMode.h"
#include "HiraganaInputMode.h"
#include "ZenKataInputMode.h"
#include "HanKataInputMode.h"
#include "HanAscInputMode.h"
#include "ZenAscInputMode.h"
#include "KanjiConversionMode.h"
#include "AsciiConvertionMode.h"

/*
  ParentInputModéÃCvbg[hA؂ւ邱Ƃ̏o钊ۃNXłB
  pRootInputModẽCX^XABIMSessionHandleBꎝɂȂ܂B
*/

class ParentInputMode : public InputMode {
  protected:
    HiraganaInputMode hiragana_input_mode;
    ZenKataInputMode zen_kata_input_mode;
    HanKataInputMode han_kata_input_mode;
    HanAscInputMode han_asc_input_mode;
    ZenAscInputMode zen_asc_input_mode;
    KanjiConversionMode kanji_conversion_mode;
    AsciiConvertionMode ascii_convertion_mode;
    bool is_kanji_mode,is_ascii_conv_mode;
    class ChildInputMode* current_input_mode;
    
    virtual void displayUnlessIgnore(const CppCFString& str);
    virtual void fixUnlessIgnore(const CppCFString& str);
    
  public:
    ParentInputMode();
    virtual ~ParentInputMode();
    
    // ParentInputModehandleInput͏falseԂB
    virtual bool handleInput(unsigned char inputChar);
    virtual bool handleBackSpace();
    virtual bool handleReturn();
    virtual bool handleArrow(int kind);
    virtual bool handleCg();
    virtual bool handleCy();
    virtual bool handleTab();
    virtual void terminate();
    
    virtual ChildInputMode& getCurrentInputMode();
    virtual ChildInputMode& getReallyCurrentInputMode(); // ϊ[hlB
    virtual HiraganaInputMode& getHiraganaInputMode();
    virtual KanjiConversionMode& getKanjiConversionMode();
    
    // ϊ[hON/OFFݒ肷B
    virtual void setEnabledKanjiMode(bool enabled);
    // AXL[ϊ[hON/OFFݒ肷B
    // ܘ_Aϊ[hƃAXL[ϊ[h̗҂ONɂȂ邱Ƃ͖B
    virtual void setEnabledAsciiConvertionMode(bool enabled);
    
    // [h
    virtual void goHiraganaInputMode();
    // SpЉ[h
    virtual void goZenKataInputMode();
    // pЉ[h
    virtual void goHanKataInputMode();
    // pAXL[[h
    virtual void goHanAscInputMode();
    // SpAXL[[h
    virtual void goZenAscInputMode();
    
    virtual bool isHiraganaInputMode() const;
    virtual bool isZenKataInputMode() const;
    virtual bool isHanKataInputMode() const;
    virtual bool isHanAscInputMode() const;
    virtual bool isZenAscInputMode() const;
    
    // ʂXVBʏ͌Ă΂ȂĂIɍXVB
    virtual void redisplay();
    
    // Mj[ACRXVBʏ͌Ă΂ȂĂIɍXVB
    virtual void updatePencilMenuIcon();
    
    //////////////////////////////////////////////////////////////////////
    // fix()ydisplay()`ChĂԂƂ͐܂BȂׂĉB
    //
    // m肷鎞ɌĂ΂Bǂ̂悤Ɋm肷邩͋̃NX肷B
    virtual void fix(const CppCFString& str_to_fix) = 0;
    //
    // \鎞ɌĂ΂Bǂ̂悤ɕ\邩͋̃NX肷B
    virtual void display(const CppCFString& str_to_display) = 0;
};
