/*  -*- objc -*-
    $Id: PreferencesController.mm,v 1.8 2003/10/30 02:26:09 phonohawk Exp $
    ---------
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <vector>
#include "CppCFData.h"

#import "PreferencesController.h"
#include "SKKServer.h"
#include "Skkserv.h"
#include "ClientConnectionFactory.h"
#include "BIMClientServer.h"

@implementation PreferencesController

+ (PreferencesController*)sharedController
{
	static PreferencesController* _shared_instance = nil;
	if (_shared_instance == nil)
	{
		_shared_instance = [[PreferencesController alloc] init];
	}
	return _shared_instance;
}

- init {
    self = [super initWithWindowNibName:@"Preferences"];
    [self window]; // [h
    
    // xύXFt[eBO
    [[self window] setLevel:NSFloatingWindowLevel];
    
    id color = [NSColor colorWithCalibratedRed:0.9 green:0.9 blue:0.9 alpha:1.0];
    NSDictionary* defDic = [NSDictionary dictionaryWithObjectsAndKeys:
					     @"~/cvs-box/skk/dic/SKK-JISYO.L", @"dic.path.main",
					 @"~/.skk-jisyo", @"dic.path.sub",
					 [NSMutableArray array], @"dic.path.kotoeri",
					 [NSArchiver archivedDataWithRootObject:color], @"dic.WindowColor.kotoeri",
					 @"0.7", @"dic.WindowAlpha.kotoeri",
					 @"5", @"dic.showCandsWindowAfterNthCand",
					 @"NO", @"dic.skk-egg-like-newline.kotoeri",
					 @"NO",@"pref.skkserv.enabled",
					 @"1178",@"pref.skkserv.port",
					 @"YES",@"pref.skkserv.local-only",
					 nil];
    
    // ~/Library/AquaSKK/΍B
    [[NSFileManager defaultManager]
	createDirectoryAtPath:[@"~/Library/AquaSKK" stringByExpandingTildeInPath]
	attributes:nil];
    
    // defaultsǂށB
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    
    [defaults registerDefaults:defDic];
    
    dicPath = [[NSMutableArray alloc] initWithArray:[defaults arrayForKey:@"dic.path.kotoeri"]];
    [winColor setColor:[NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"dic.WindowColor.kotoeri"]]];
    [winTransparent setDoubleValue:[defaults floatForKey:@"dic.WindowAlpha.kotoeri"]];
    [skkEggLikeNewlineBtn setState:[defaults boolForKey:@"dic.skk-egg-like-newline.kotoeri"]];
    
    path_to_main_dic = [[NSString alloc] initWithString:[defaults stringForKey:@"dic.path.main"]];
    path_to_sub_dic = [[NSString alloc] initWithString:[defaults stringForKey:@"dic.path.sub"]];
    
    // skkserv
    [skkserv_enabled setState:[defaults boolForKey:@"pref.skkserv.enabled"]];
    [skkserv_port setIntValue:[defaults integerForKey:@"pref.skkserv.port"]];
    [skkserv_local_only setState:[defaults boolForKey:@"pref.skkserv.local-only"]];
    
    return self;
}

- (IBAction)saveUserDefault:(id)sender {
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    
    [defaults setObject:dicPath forKey:@"dic.path.kotoeri"];
    
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[winColor color]] forKey:@"dic.WindowColor.kotoeri"];
    [defaults setFloat:[winTransparent doubleValue] forKey:@"dic.WindowAlpha.kotoeri"];
    [defaults setBool:[skkEggLikeNewlineBtn state] forKey:@"dic.skk-egg-like-newline.kotoeri"];

    // skkserv
    [defaults setBool:[skkserv_enabled state] forKey:@"pref.skkserv.enabled"];
    [defaults setInteger:[skkserv_port intValue] forKey:@"pref.skkserv.port"];
    [defaults setBool:[skkserv_local_only state] forKey:@"pref.skkserv.local-only"];
    
    [defaults synchronize];
}


- (NSColor *)windowColor
{
    return [winColor color];
}

- (float)windowAlpha
{
    return [winTransparent doubleValue];
}

- (BOOL)isSkkEggLikeNewline
{
    return [skkEggLikeNewlineBtn state];
}

- (IBAction)skkEggLikeNewline:(id)sender
{
    bool skkegglikenewline = ( [sender state] ) ? true : false;
    
	CppCFData bool_data(&skkegglikenewline,sizeof(bool));
    ClientConnectionFactory::sharedFactory().newConnection().send(kBasicMessageIsSkkEggLikeNewline,bool_data);
    
    [self saveUserDefault:self];
}

// SKK

- (NSString*)getPathToMainDic
{
	return [path_to_main_dic stringByExpandingTildeInPath];
}

- (NSString*)getPathToSubDic
{
	return [path_to_sub_dic stringByExpandingTildeInPath];
}

- (NSString*)getPathToUserDic
{
	return [@"~/Library/AquaSKK/skk-user-dic" stringByExpandingTildeInPath];
}

- (IBAction)showWindow:(id)sender
{
	[maindic_path setStringValue:path_to_main_dic];
	[subdic_path setStringValue:path_to_sub_dic];
        
        [NSApp activateIgnoringOtherApps:YES];
        [[self window] makeKeyAndOrderFront:nil];
	
	[super showWindow:sender];
}

- (BOOL)windowShouldClose:(NSNotification *)aNotification {
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    
    if (![[[maindic_path stringValue] stringByAbbreviatingWithTildeInPath]
	     isEqualToString:path_to_main_dic]) {
	// C̏ꏊς
	[path_to_main_dic release];
	path_to_main_dic = [[[maindic_path stringValue] stringByAbbreviatingWithTildeInPath] retain];
	
	[defaults setObject:path_to_main_dic forKey:@"dic.path.main"];
	
	SKKServer::sharedServer().getBaseDic().changeDictionaryFile([[self getPathToMainDic] cString]);
    }
    
    if (![[[subdic_path stringValue] stringByAbbreviatingWithTildeInPath]
	     isEqualToString:path_to_sub_dic]) {
	// Tȕꏊς
	[path_to_sub_dic release];
	path_to_sub_dic = [[[subdic_path stringValue] stringByAbbreviatingWithTildeInPath] retain];
	
	[defaults setObject:path_to_sub_dic forKey:@"dic.path.sub"];
	
	SKKServer::sharedServer().getSubDic().changeDictionaryFile([[self getPathToSubDic] cString]);
    }
    
    Skkserv& skkserv = Skkserv::sharedServer();
    skkserv.setLocalOnly([skkserv_local_only state] == YES ? true : false);
    if (skkserv.isRunning()) {
	if ([skkserv_enabled state] == NO) {
	    // ~
	    skkserv.stop();
	}
	else if ([skkserv_port intValue] != skkserv.port()) {
	    // |[g̍X
	    skkserv.stop();
	    skkserv.start([skkserv_port intValue]);
	}
    }
    else {
	if ([skkserv_enabled state] == YES) {
	    // Jn
	    skkserv.start([skkserv_port intValue]);
	}
    }
    
    [self saveUserDefault:self];
    return TRUE;
}

- (IBAction)selectMainDic:(id)sender
{
}

- (IBAction)selectSubDic:(id)sender
{
}

// Ƃ莫

- (IBAction)addKotoeriDic:(id)sender
{
    NSOpenPanel *op = [NSOpenPanel openPanel];

    [op beginSheetForDirectory:NSHomeDirectory()
                                file:nil
                                types:nil
                                modalForWindow:[self window]
                                modalDelegate:self
                                didEndSelector:@selector(openPanelDidEnd:returnCode:contextInfo:)
                                contextInfo:NULL];
                                
    [kotoeriDicTable reloadData];
}

- (void)openPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
    if( returnCode == NSOKButton ){
        
        if( [dicPath containsObject:[sheet filename]] == YES ){
            NSBeep();
            return;
        }
        
        bool b = SKKServer::sharedServer().getKotoeriDic().checkDictionary( (CFStringRef)[sheet filename] );
        
        if( b == false ){
            NSBeep();
            return;
        }
        
        [dicPath addObject:[sheet filename]];
        
        [self controlSetting];
        [kotoeriDicTable reloadData];
    }
}

- (IBAction)removeKotoeriDic:(id)sender
{
    int n = [kotoeriDicTable selectedRow];
    
    if( n == -1 )
        return;
    
    [dicPath removeObjectAtIndex:n];
    
    [self controlSetting];
    [kotoeriDicTable reloadData];
}

- (void)controlSetting
{
    [addKotoeriBtn setEnabled:( [dicPath count] == 10 ) ? NO : YES ];
    [removeKotoeriBtn setEnabled:( [dicPath count] == 0 ) ? NO : YES ];
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [dicPath count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    // Second column
	if([[aTableColumn identifier] isEqualToString:@"name"])
        return [dicPath objectAtIndex:rowIndex];
    
    // Not reach this statement
    return nil;
}

- (NSArray *)path
{
    return dicPath;
}

// skkserv
- (BOOL)isSkkservEnabled {
    return [skkserv_enabled state];
}

- (int)skkservPort {
    return [skkserv_port intValue];
}

- (BOOL)isSkkservLocalOnly {
    return [skkserv_local_only state];
}

@end
