/*
	$Id: SKKDictionary.h,v 1.11 2002/11/29 12:23:11 phonohawk Exp $
	---------
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#pragma once
#include <string>
#include <iostream>
#include <map>
//#include <hash_map>
#include <vector>
#include "CppCFString.h"
#include "OkuriganaEntry.h"
#include "Dictionary.h"
using namespace std;

class SKKDictionary : public Dictionary
{
	protected:
		bool read_only;
		const string fpath;
		map<CppCFString,vector<OkuriganaEntry>*> okuri_ari_table;
		map<CppCFString,vector<CppCFString>*> okuri_nasi_table;
		//hash_map<CppCFString,vector<OkuriganaEntry>*,hash_CppCFString,equal_to<CppCFString> > okuri_ari_table;
		//hash_map<CppCFString,vector<CppCFString>*,hash_CppCFString,equal_to<CppCFString> > okuri_nasi_table;
		vector<string> okuri_ari_unparsed; // p[XO͕R[hFĂȂB
		vector<string> okuri_nasi_unparsed;
		
		virtual void load(istream& is);
		virtual void save(ostream& ofs,bool& terminate_flag);
		
	public:
		SKKDictionary(const string& fpath,bool read_only);
		virtual ~SKKDictionary();
		
		virtual const string& getFilePath() const;
		
		virtual vector<CppCFString> findWithoutOkuri(const CppCFString& query);
		virtual vector<OkuriganaEntry> findWithOkuri(const CppCFString& root);
		
		virtual vector<CppCFString> findCompletions(const CppCFString& query);
		
		virtual void registerWord(const CppCFString& index,const CppCFString& okuri,const CppCFString& kanji);
		virtual void registerWord(const CppCFString& index,const CppCFString& kanji);
		virtual void removeWord(const CppCFString& index,const CppCFString& okuri,const CppCFString& kanji);
		virtual void removeWord(const CppCFString& index,const CppCFString& kanji);
		
		virtual void changeDictionaryFile(const string& fpath); // p[XŝŎԂB
		
		virtual void parseAll(); // SKKDictionaryĝ݂ĂԎB
		virtual void save(bool& terminate_flag); // TuXbhĂԂƁB
};

void parseOkuriAriEntry(const CppCFString& line,CppCFString& index,vector<OkuriganaEntry>& entries);
void parseOkuriNasiEntry(const CppCFString& line,CppCFString& index,vector<CppCFString>& entries);