/*
	$Id: ServerConnectionFactory.cpp,v 1.3 2002/11/29 12:29:10 phonohawk Exp $
	---------
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <Carbon/Carbon.h>
#include "ServerConnectionFactory.h"
#include "BIMClientServer.h"

static ServerConnectionFactory* _shared_instance = NULL;

ServerConnectionFactory& ServerConnectionFactory::sharedFactory()
{
	if (_shared_instance == NULL)
	{
		_shared_instance = new ServerConnectionFactory();
	}
	return *_shared_instance;
}

ServerConnectionFactory::ServerConnectionFactory()
	:serverIsRunning(false)
{
	
}

ServerConnectionFactory::~ServerConnectionFactory()
{

}

void ServerConnectionFactory::waitUntilServerStarts()
{
	prepareServer();
	
	if (!serverIsRunning)
	{
		// Iڑ\ɂȂ܂ő҂B40b҂Ăڑ\ɂȂȂ΃^CAEgB
		CppCFString server_port_name(kBasicServerPortName);
		
		unsigned long timeout = TickCount() + (40 * 60);
		while(true)
		{
			try
			{
				ServerConnection connection(server_port_name);
				// ONȂ΃RlNVꂽƂɂȂ̂ŔB
				serverIsRunning = true;
				break;
			}
			catch (std::exception& e)
			{
				if(TickCount() > timeout)
				{
					fprintf(stderr,"*** Timed out. AquaSKKServer.app didn\'t get online.\n");
					break; // ^CAEg
				}
				else
				{
					// 0.3b҂čĐڑ
					usleep(300000); // 0.3 * 1000 * 1000
				}
			}
		}
	}
}

ServerConnectionFactory& ServerConnectionFactory::prepareServer()
{
	serverIsRunning = false;
	
	// Ĩ|[gJĂ݂āAJȂɋNĂ̂Ŗ߂B
    CFMessagePortRef serverPortRef = CFMessagePortCreateRemote(NULL,CFSTR(kBasicServerPortName));
    if(serverPortRef)
	{
		serverIsRunning = true;
		
        CFRelease(serverPortRef);
        return *this;
    }
	
	// ohʎqAIM̃oh𓾂B
    CFBundleRef myComponentBundle = CFBundleGetBundleWithIdentifier(CFSTR(kIMBundleIdentifier));
    
    // SharedSupportfBNg擾
    CFURLRef sharedSupportURL = CFBundleCopySharedSupportURL(myComponentBundle);
    
    // Iւ̃pXURLƂč쐬
	CFURLRef serverURL = CFURLCreateCopyAppendingPathComponent(NULL,sharedSupportURL,
                                                           CFSTR(kCSTR_SKKServerName),FALSE);
    
    // URLFSRefɕϊAFSSpecɕϊB
	FSRef serverFSRef;
    CFURLGetFSRef(serverURL,&serverFSRef);
	
	FSSpec serverFileSpec;
	FSGetCatalogInfo(&serverFSRef,kFSCatInfoNone,NULL,NULL,&serverFileSpec,NULL);
	
	// E`
    LaunchParamBlockRec		launchParams;
    launchParams.launchBlockID = extendedBlock;
    launchParams.launchEPBLength = extendedBlockLen;
    launchParams.launchFileFlags = 0;
    launchParams.launchControlFlags = launchNoFileFlags + launchContinue + launchDontSwitch;
    launchParams.launchAppSpec = &serverFileSpec;
    launchParams.launchAppParameters = NULL;
    LaunchApplication(&launchParams);
	
    if(serverURL) CFRelease(serverURL);
    if(sharedSupportURL) CFRelease(sharedSupportURL);
	return *this;
}

ServerConnection ServerConnectionFactory::newConnection()
{
	try
	{
		return ServerConnection(kBasicServerPortName);
	}
	catch (std::exception& e)
	{
		// INĂȂB
		waitUntilServerStarts();
		return ServerConnection(kBasicServerPortName);
	}
}