/*
	$Id: ServerMessageReceiver.mm,v 1.3 2002/11/29 12:23:11 phonohawk Exp $
	---------
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#import <AppKit/AppKit.h>
#import "CandidatesManager.h"
#import "CandidatesWindowController.h"
#import "PreferencesController.h"
#import "UserDicSaver.h"
#include "ServerMessageReceiver.h"
#include "BIMClientServer.h"
#include "CppCFString.h"
#include "CppCFData.h"
#include "SKKServer.h"
#include "ClientConnectionFactory.h"

static ServerMessageReceiver* _shared_instance = NULL;

void ServerMessageReceiver::start(CFRunLoopRef runloop_to_install)
{
	if (_shared_instance == NULL)
	{
		_shared_instance = new ServerMessageReceiver(runloop_to_install);
	}
}

ServerMessageReceiver::ServerMessageReceiver(CFRunLoopRef runloop_to_install) throw(std::exception)
	:CppMessagePortServer(kBasicServerPortName,runloop_to_install)
{
	
}

ServerMessageReceiver::ServerMessageReceiver(const CppCFString& port_name,CFRunLoopRef runloop_to_install) throw(std::exception)
	:CppMessagePortServer(port_name,runloop_to_install)
{
	// ̃RXgN^͎gp֎~
	throw std::exception();
}

ServerMessageReceiver::~ServerMessageReceiver()
{

}

CppCFData ServerMessageReceiver::messageReceived(SInt32 message_id,const CppCFData& attachment)
{
	CppCFData reply;
	
	// wb_oBNCAgPSNĂB
	BasicMessageHeader header;
	if (attachment.getLength() > 0)
	{
		attachment.copyData(0,sizeof(BasicMessageHeader),&header);
	}
	
	CppCFData body;
	if (attachment.getLength() > sizeof(BasicMessageHeader))
	{
		body = attachment.subdata(sizeof(BasicMessageHeader));
	}
	
	switch (message_id)
	{
        // NCAgANeBuɂȂB
		case kBasicMessageActivated:
			// ANeBuȃNCAgƂĕۑĂB
			ClientConnectionFactory::sharedFactory().setDefaultTarget(header.fProcessSerialNumber);
			break;
		
		// NCAgANeBułȂȂB
        case kBasicMessageDeactivated:
		{
			ProcessSerialNumber psn;
            psn.highLongOfPSN = 0;
            psn.lowLongOfPSN = kNoProcess;
			
			ClientConnectionFactory::sharedFactory().setDefaultTarget(psn);
		}
		break;
		
		// pbgB悤ATSMwB
        case kBasicMessageHidePalettes:
			break;
		
		case kSKKMessageSearch:
			reply = searchWord(body);
			break;
		
		case kSKKCreateCandidatesWindow:
			reply = createCandidatesWindow(body);
			break;
		
		case kSKKCandidatesWindowNext:
			candidatesWindowNext(body);
			break;
		
		case kSKKCandidatesWindowPrev:
			candidatesWindowPrev(body);
			break;
		
		case kSKKCloseCandidatesWindow:
			closeCandidatesWindow(body);
			break;
		
		case kSKKRegisterThisToUserDic:
			registerThisToUserDic(body);
			break;
		
		case kSKKRemoveThisFromUserDic:
			removeThisFromUserDic(body);
			break;
		
		case kSKKShowAboutBox:
			showAboutBox();
			break;
		
		case kSKKShowPreferencesBox:
			showPreferencesBox();
			break;
        
        case kBasicMessageIsSkkEggLikeNewline:
            reply = eggLikeNewline();
			break;
        
		case kSKKShowWordRegistrationBox:
			break;
		
		case kSKKFetchCompletions:
			reply = fetchCompletions(body);
			break;
            
        default:
            break;
    }
	
	return reply;
}

CppCFData ServerMessageReceiver::searchWord(const CppCFData& attachment)
{
	// ǉf[^ƂUniChar̔zB
	CppCFString query(attachment.getData());
	CppCFString reply(SKKServer::sharedServer().search(query));
	
	return CppCFData(reply.toCFData());
}

CppCFData ServerMessageReceiver::createCandidatesWindow(const CppCFData& attachment)
{
	// ǉf[^ƂQuickDrawWUniChar̔zB
	int qd_x = attachment.getSInt16(0);
	int qd_y = attachment.getSInt16(2);
	CppCFString candidates(attachment.subdata(4).getData());
	
	CandidatesManager& cm = CandidatesManager::sharedManager();
	unsigned window_id = cm.createAndShow(qd_x,qd_y,candidates);
	
	CandidatesWindowController* cwc = cm.getWindowController(window_id);
	unsigned cands_per_frame = [cwc getCandidatesPerFrame];
	unsigned num_of_frames = [cwc getNumOfFrames];
	
	CppCFData reply;
	reply.append(static_cast<UInt16>(window_id));
	reply.append(static_cast<UInt16>(cands_per_frame));
	reply.append(static_cast<UInt16>(num_of_frames));
	return reply;
}

void ServerMessageReceiver::candidatesWindowNext(const CppCFData& attachment)
{
	// ǉf[^ƂwindowID𓾂B
	CandidatesManager::sharedManager().windowNext(attachment.getUInt16(0));
}

void ServerMessageReceiver::candidatesWindowPrev(const CppCFData& attachment)
{
	CandidatesManager::sharedManager().windowPrev(attachment.getUInt16(0));
}

void ServerMessageReceiver::closeCandidatesWindow(const CppCFData& attachment)
{
	CandidatesManager::sharedManager().closeWindow(attachment.getUInt16(0));
}

void ServerMessageReceiver::registerThisToUserDic(const CppCFData& attachment)
{
	CppCFString query(attachment.getData());
	
	SKKServer::sharedServer().registerToUserDic(query);
	[[UserDicSaver sharedSaver] userDicGotDirty];
}

void ServerMessageReceiver::removeThisFromUserDic(const CppCFData& attachment)
{
	CppCFString query(attachment.getData());
	
	SKKServer::sharedServer().removeFromUserDic(query);
	[[UserDicSaver sharedSaver] userDicGotDirty];
}

void ServerMessageReceiver::showAboutBox()
{
	static NSWindowController* _static_controller = nil;
	if (_static_controller == nil)
		_static_controller = [[NSWindowController alloc] initWithWindowNibName:@"AboutBox"];
	
	[[_static_controller window] setLevel:NSFloatingWindowLevel];
	[_static_controller showWindow:nil];
}

void ServerMessageReceiver::showPreferencesBox()
{
	[[PreferencesController sharedController] showWindow:nil];
}

CppCFData ServerMessageReceiver::eggLikeNewline()
{
	bool isSkkEggLikeNewline =
			[[PreferencesController sharedController] isSkkEggLikeNewline] ? true : false;
	
	return CppCFData(&isSkkEggLikeNewline,sizeof(bool));
}

CppCFData ServerMessageReceiver::fetchCompletions(const CppCFData& attachment)
{
	CppCFString query(attachment.getData());
	CppCFString reply(SKKServer::sharedServer().searchCompletions(query));
	
	return CppCFData(reply.toCFData());
}