/*
	$Id: WordRegisterMode.cpp,v 1.12 2003/10/30 02:26:09 phonohawk Exp $
	---------
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "WordRegisterMode.h"
#include "KanjiConversionMode.h"
#include "BIM.h"
#include "SkkConfig.h"
#include "CppScrap.h"

WordRegisterMode::WordRegisterMode(ChildInputMode& owner,const CppCFString& word)
	:ParentInputMode(),
	 owner(&owner),
	 caret(0)
{
	// [o^:word]
	title.append('[').append(0x767b).append(0x9332).append(':').append(word).append(']');
	
	BIMSessionHandle session_handle = BIMGetActiveSession();
	imsession_input_mode = (*session_handle)->imsession_input_mode;
}

bool WordRegisterMode::handleBackSpace()
{
	if (getReallyCurrentInputMode().handleBackSpace())
	{
		// really current input modetrueԂ牽trueԂB
		redisplay();
		return true;
	}
	
	ChildInputMode& input_mode = getReallyCurrentInputMode();
	
	if (!input_mode.handleBackSpace())
	{
		// Lbg̈ʒuOɕΈꕶ폜BΖB
		if (caret > 0)
		{
			buffer.eraseCharAt(caret-1);
			caret--;
		}
	}
	//redisplay();
	display(CppCFString());
	return true; // ǂɂł{͏ȂB
}

bool WordRegisterMode::handleReturn()
{
	ChildInputMode& input_mode = getReallyCurrentInputMode();
	
	if (!input_mode.handleReturn())
	{
		fix(input_mode.getStringToFix());
		
		if (buffer.length() > 0)
		{
			(dynamic_cast<RegistrationStarter*>(owner))->wordRegistrationFinished(); // Po^IB
		}
		else
		{
			(dynamic_cast<RegistrationStarter*>(owner))->wordRegistrationCanceled(); // Po^fB
		}
	}
	return true;
}

bool WordRegisterMode::handleArrow(int kind)
{
	if (getReallyCurrentInputMode().handleArrow(kind))
	{
		// really current input modetrueԂ牽trueԂB
		return true;
	}
	
	// ォ͖BE̓LbgړB
	if (kind == ARROW_LEFT && caret > 0)
	{
		caret--;
	}
	else if (kind == ARROW_RIGHT && caret < buffer.length())
	{
		caret++;
	}
	display(CppCFString());
	return true;
}

bool WordRegisterMode::handleCg()
{
	if (getReallyCurrentInputMode().handleCg())
	{
		// really current input modetrueԂ牽trueԂB
		redisplay();
		return true;
	}
	
	getReallyCurrentInputMode().terminate();
	
	// bufferɂB
	buffer.clear();
	caret = 0;
	(dynamic_cast<RegistrationStarter*>(owner))->wordRegistrationCanceled(); // Po^fB
	return true;
}

bool WordRegisterMode::handleCy()
{
	if (getReallyCurrentInputMode().handleCy())
	{
		// really current input modetrueԂ牽trueԂB
		return true;
	}
	
	CppScrapFlavor* clipped = CppScrap().makeFlavor(kScrapFlavorTypeUnicode);
	if (clipped->exists()) {
		// Nbv{[hɕ񂪓Ă̂ŁALbg̈ʒuɑ}B
		// Lbg͑}ꂽ̎ɈړB
		CppCFString str_in_clip(clipped->content().getData());
		buffer.insert(caret,str_in_clip);
		caret += str_in_clip.length();
		
		display(CppCFString());
	}
        delete clipped;
	return true;
}

bool WordRegisterMode::handleTab()
{
	ParentInputMode::handleTab();
	return true; // ɐHB
}

CppCFString WordRegisterMode::getContent() const
{
	return buffer;
}

void WordRegisterMode::redisplay()
{
	ChildInputMode& input_mode = getReallyCurrentInputMode();
	
	CppCFString str_to_display = input_mode.getStringToDisplay();
	if (str_to_display.length() == 0 ||
		(str_to_display.length() > 0 && str_to_display[0] != kIgnoreThis)) // kIgnoreThisȂXVȂB
	{
		display(str_to_display);
	}
}

void WordRegisterMode::fix(const CppCFString& str_to_fix)
{
	if (str_to_fix.length() > 0 &&
		str_to_fix[0] == kIgnoreThis) return;
	
	// obt@̃Lbg̈ʒuɑ}B
	buffer.insert(caret,str_to_fix);
	
	// Lbg}AEɈړB
	caret += str_to_fix.length();
	
	imsession_input_mode->display(CppCFString(title).append('[')
		.append(buffer.substring(0,caret)).append(kCaret)
		.append(buffer.substring(caret)).append(']'));
}

void WordRegisterMode::display(const CppCFString& str_to_display)
{
	if (str_to_display.length() > 0 &&
		str_to_display[0] == kIgnoreThis) return;
		
	// \F
	// [*][]
	imsession_input_mode->display(CppCFString(title).append('[')
		.append(buffer.substring(0,caret)).append(str_to_display).append(kCaret)
		.append(buffer.substring(caret)).append(']'));
}